/* Viagra IRCd - OvErRiTe
 *
 * The below code is based on the logging system included with Epona
 * services.  It is an excellent logging implementation and therefore
 * adapted for use in Viagra IRCd.
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free softwmare; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/time.h>
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "zlink.h"
#include <sys/stat.h>
#include <fcntl.h>
#if !defined(ULTRIX) && !defined(SGI) && !defined(sequent) && !defined(__convex__)
#include <sys/param.h>
#endif
#include <time.h>
#include "h.h"
#include "fdlist.h"

/* in order to include a complete logging system I have decided to break
 * logging up into two parts; ircd (system) log and an ircd (oper) log.
 * the system log will contain ircd operational logging while the oper log
 * will contain commands issued by opers. -OvErRiTe
 */
 
/* system */
static FILE *ircd_logfile;
static int ircd_curday = 0;
/* oper */
static FILE *oper_logfile;
static int oper_curday = 0;

/*************************************************************************/
/*  Below are the logging routines for the ircd (system)                 */
/*  This logs ircd messages to their own file for ease of reading        */
/*************************************************************************/
static int ircd_get_logname(char *name, int count, struct tm *tm) {

	char timestamp[32];

	if (!tm) {
		time_t t;

		time(&t);
		tm = localtime(&t);
	}

	strftime(timestamp, count, "%m-%d-%Y", tm);
	snprintf(name, count, "%s_%s.log", SYSTEMPATH, timestamp);
	ircd_curday = tm->tm_yday;

	return 1;
}

static void ircd_remove_log(void) {
	time_t t;
	struct tm tm;

	char name[PATH_MAX];

	time(&t);
	t -= (60*60*24*KEEPLOGS);
	tm = *localtime(&t);

	if (!ircd_get_logname(name, sizeof(name), &tm)) return;
	unlink(name);
}

static void ircd_checkday(void) {
	time_t t;
	struct tm tm;

	time(&t);
	tm = *localtime(&t);

	if (ircd_curday != tm.tm_yday) {
		ircd_close_log();
		ircd_remove_log();
		ircd_open_log();
	}
}

int ircd_open_log(void)
{
	char name[PATH_MAX];

    if (ircd_logfile)
	return 0;

	if (!ircd_get_logname(name, sizeof(name), NULL)) return 0;
    ircd_logfile = fopen(name, "a");

    if (ircd_logfile)
	setbuf(ircd_logfile, NULL);
    return ircd_logfile!=NULL ? 0 : -1;
}

void ircd_close_log(void)
{
    if (!ircd_logfile)
	return;
    fclose(ircd_logfile);
    ircd_logfile = NULL;
}

void ircd_log(char *fmt, ...)
{
    va_list args;
    time_t t;
    struct tm tm;
    char buf[256];

    ircd_checkday();

    if (!fmt) {
        return; 
    }

    va_start(args, fmt);
    time(&t);
    tm = *localtime(&t);

	strftime(buf, sizeof(buf)-1, "[%b %d %H:%M:%S %Y] ", &tm);

    if (ircd_logfile) {
	fputs(buf, ircd_logfile);
	vfprintf(ircd_logfile, fmt, args);
	fputc('\n', ircd_logfile);
    }

    va_end(args);
}
/*************************************************************************/
/* End ircd (system) log                                                 */
/*************************************************************************/

/*************************************************************************/
/*  Below are the logging routines for the ircd (oper) logs              */
/*  This logs oper messages to their own file for ease of reading        */
/*************************************************************************/
static int oper_get_logname(char *name, int count, struct tm *tm) {

	char timestamp[32];

	if (!tm) {
		time_t t;

		time(&t);
		tm = localtime(&t);
	}

	strftime(timestamp, count, "%m-%d-%Y", tm);
	snprintf(name, count, "%s_%s.log", OPERPATH, timestamp);
	oper_curday = tm->tm_yday;

	return 1;
}

static void oper_remove_log(void) {
	time_t t;
	struct tm tm;

	char name[PATH_MAX];

	time(&t);
	t -= (60*60*24*KEEPLOGS);
	tm = *localtime(&t);

	if (!oper_get_logname(name, sizeof(name), &tm)) return;
	unlink(name);
}

static void oper_checkday(void) {
	time_t t;
	struct tm tm;

	time(&t);
	tm = *localtime(&t);

	if (oper_curday != tm.tm_yday) {
		oper_close_log();
		oper_remove_log();
		oper_open_log();
	}
}

int oper_open_log(void)
{
	char name[PATH_MAX];

    if (oper_logfile)
	return 0;

	if (!oper_get_logname(name, sizeof(name), NULL)) return 0;
    oper_logfile = fopen(name, "a");

    if (oper_logfile)
	setbuf(oper_logfile, NULL);
    return oper_logfile!=NULL ? 0 : -1;
}

void oper_close_log(void)
{
    if (!oper_logfile)
	return;
    fclose(oper_logfile);
    oper_logfile = NULL;
}

void oper_log(char *fmt, ...)
{
    va_list args;
    time_t t;
    struct tm tm;
    char buf[256];

    oper_checkday();

    if (!fmt) {
        return; 
    }

    va_start(args, fmt);
    time(&t);
    tm = *localtime(&t);

	strftime(buf, sizeof(buf)-1, "[%b %d %H:%M:%S %Y] ", &tm);

    if (oper_logfile) {
	fputs(buf, oper_logfile);
	vfprintf(oper_logfile, fmt, args);
	fputc('\n', oper_logfile);
    }

    va_end(args);
}
/*************************************************************************/
/* End ircd (oper) log                                                   */
/*************************************************************************/
